library IEEE;
use IEEE.std_logic_1164.all;
use ieee.std_logic_textio.all;
use IEEE.numeric_std.all;

library std;
use std.textio.all;

entity Lab2TestBench is
  
end Lab2TestBench;

architecture Lab2_arch of Lab2TestBench is


  -----------------------------------------------------------------------
  -- component declarations
  -----------------------------------------------------------------------
  component SC_HEX is
    port (	
      sc: in std_logic_vector(7 downto 0);
      prs : in std_logic;
      hex : out std_logic_vector(3 downto 0)
      );
  end component SC_HEX;
  component PARITY is
    port (
      HEX: in STD_LOGIC_VECTOR (3 downto 0);
      LED: out STD_LOGIC_VECTOR (6 downto 0)
      );
  end component PARITY;
  -----------------------------------------------------------------------

    -----------------------------------------------------------------------
    -- signal/constant declarations
    -----------------------------------------------------------------------
  signal scancode: std_logic_vector(7 downto 0);
  signal parity_led: std_logic_vector(6 downto 0);
  signal clk : std_logic := '0';
  signal prs_signal: std_logic := '1';
  signal hex_signal : std_logic_vector(3 downto 0);
  
  constant sim_time : time := 170 ns;
  constant clk_period : time := 5 ns;
  signal done : boolean := false;
  signal failed_h, failed_p : integer := 0;
  
begin
  -----------------------------------------------------------------------------
  -- component instantiations
  -----------------------------------------------------------------------------
  kb_hex_mod: SC_HEX
    port map (sc=>scancode, prs=>prs_signal, hex=>hex_signal);
  parity_mod: PARITY
    port map (HEX=>hex_signal, LED=>parity_led);

-- the clock upon which we synchronize all of our processes
  clk <= not clk after clk_period when not done else '0';
  
-------------------------------------------------------------------------------
-- TestProcess: Supplies the input to the dut
-------------------------------------------------------------------------------
         TestProcess: process(clk)
           variable li : line;
           file input : text is "input.txt";
           variable scancode_v : std_logic_vector(7 downto 0);
         begin
	   prs_signal <= clk;
	   if clk'event and clk = '1' then
             if not endfile(input) then
               readline(input, li);
               hread(li, scancode_v);
               scancode <= scancode_v;
             else
               done <= true;
             end if;
	   end if;
         end process; 

-------------------------------------------------------------------------------
-- VerifyProcess: Checks the output against the expected output
-------------------------------------------------------------------------------
           VerifyProcess: process(clk)
             variable li1, li2, lout : line;
             file output_hex : text is "output_hex.txt";
             file output_par : text is "output_par.txt";
             variable hex_v : std_logic_vector(3 downto 0);
             variable par : std_logic;
           begin
             if clk'event and clk = '0' then  -- other edge
               if not endfile(output_hex) then
                 readline(output_hex, li1);
                 read(li1, hex_v);
                 if not (hex_v = hex_signal) then
                   failed_h <= failed_h + 1;
                   assert (false)
                     report "Hex signals do not match" severity error;
                   write(lout, string'("At time "));
                   write(lout, now);
                   write(lout, string'(", "));
                   write(lout, hex_signal);
                   write(lout, string'(" should be "));
                   write(lout, hex_v);
                   writeline(output, lout);
                 end if;
               end if;
               if not endfile(output_par) then
                 readline(output_par, li2);
                 read(li2, par);
                 case par is                       
                   when '0' =>
                     if not (parity_led="1110111") then
                       failed_p <= failed_p + 1;
                       assert (false)
                         report "Parity does not match" severity error;
                     end if;
                   when '1' =>
                     if not (parity_led="0010010") then
                       failed_p <= failed_p + 1;
                       assert (false)
                         report "Parity does not match" severity error;
                     end if;
                   when others => assert(false) report "Unexpected parity output"
                                    severity failure;
                 end case;
               end if;
             end if;
           end process;	

  -----------------------------------------------------------------------------
  -- ReportProcess: outputs a report of the behaviour of the design
  -----------------------------------------------------------------------------
  ReportProcess: process(done)
    variable lout : line;
  begin
    if done then
      write(lout, string'("Failed hex check "));
      write(lout, failed_h);
      write(lout, string'(" times."));
      writeline(output, lout);
      write(lout, string'("Failed parity check "));
      write(lout, failed_p);
      write(lout, string'(" times."));
      writeline(output, lout);
    end if;
  end process;
  
end Lab2_arch;








